"""
The ultimate KEGS webscraping experience.
"""
from __future__ import annotations

from . import bromcom, it, oliver, papercut, site, util, vle, kerboodle, activelearn, ms365
from .util import exceptions
